# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

from inspire_utils.dedupers import dedupe_list, dedupe_list_of_dicts, dedupe_all_lists


def test_dedupe_list():
    list_with_duplicates = ['foo', 'bar', 'foo']

    expected = ['foo', 'bar']
    result = dedupe_list(list_with_duplicates)

    assert expected == result


def test_dedupe_list_of_dicts():
    list_of_dicts_with_duplicates = [
        {'a': 123, 'b': 1234},
        {'a': 3222, 'b': 1234},
        {'a': 123, 'b': 1234},
    ]

    expected = [{'a': 123, 'b': 1234}, {'a': 3222, 'b': 1234}]
    result = dedupe_list_of_dicts(list_of_dicts_with_duplicates)

    assert expected == result


def test_dedupe_all_lists():
    obj = {
        "l0": list(range(10)) + list(range(10)),
        "o1": [{"foo": "bar"}] * 10,
        "o2": [{"foo": [1, 2]}, {"foo": [1, 1, 2]}] * 10,
    }

    expected = {"l0": list(range(10)), "o1": [{"foo": "bar"}], "o2": [{"foo": [1, 2]}]}

    assert dedupe_all_lists(obj) == expected


def test_dedupe_all_lists_honors_exclude_keys():
    obj = {
        "l0": list(range(10)) + list(range(10)),
        "o1": [{"foo": "bar"}] * 10,
        "o2": [{"foo": [1, 2]}, {"foo": [1, 1, 2]}] * 10,
    }

    expected = {
        "l0": list(range(10)),
        "o1": [{"foo": "bar"}] * 10,
        "o2": [{"foo": [1, 2]}],
    }

    assert dedupe_all_lists(obj, exclude_keys=["o1"]) == expected


def test_dedupe_all_lists_dedupes_under_excluded_keys():
    obj = {
        "l0": list(range(10)) + list(range(10)),
        "o1": [{"foo": "bar"}] * 10,
        "o2": [{"foo": [1, 2]}, {"foo": [1, 1, 2]}] * 10,
    }

    expected = {
        "l0": list(range(10)),
        "o1": [{"foo": "bar"}],
        "o2": [{"foo": [1, 2]}] * 20,
    }

    assert dedupe_all_lists(obj, exclude_keys=["o2"]) == expected



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

from inspire_utils.helpers import (
    force_list,
    maybe_float,
    maybe_int,
    remove_tags,
)


def test_force_list_returns_empty_list_on_none():
    expected = []
    result = force_list(None)

    assert expected == result


def test_force_list_wraps_strings_in_a_list():
    expected = ['foo']
    result = force_list('foo')

    assert expected == result


def test_force_list_converts_tuples_to_lists():
    expected = ['foo', 'bar', 'baz']
    result = force_list(('foo', 'bar', 'baz'))

    assert expected == result


def test_force_list_does_not_touch_lists():
    expected = ['foo', 'bar', 'baz']
    result = force_list(['foo', 'bar', 'baz'])

    assert expected == result


def test_maybe_float_returns_float_if_possible():
    expected = 35.0499505
    result = maybe_float('35.0499505')

    assert expected == result


def test_maybe_float_returns_none_otherwise():
    assert maybe_float(('35.0499505', '137.052276')) is None


def test_maybe_int_returns_int_if_possible():
    expected = 10
    result = maybe_int('10')

    assert expected == result


def test_maybe_int_returns_none_otherwise():
    assert maybe_int('216+337') is None


def test_remove_tags_allowed_trees_strip():
    allowed_trees = ('b',)
    strip = '@class="hidden"'
    snippet = '<p><b><i>Only</i></b> this text remains.<span class="hidden">Not this one.</span></p>'

    result = remove_tags(snippet, allowed_trees=allowed_trees, strip=strip)
    expected = u'<b><i>Only</i></b> this text remains.'

    assert result == expected


def test_remove_tags_allowed_tags_strip():
    allowed_tags = ('b',)
    strip = '@class="hidden"'
    snippet = '<p><b><i>Only</i></b> this text remains.<span class="hidden">Not this one.</span></p>'

    result = remove_tags(snippet, allowed_tags=allowed_tags, strip=strip)
    expected = u'<b>Only</b> this text remains.'

    assert result == expected


def test_remove_tags_allowed_tags_strip_preserves_text():
    allowed_tags = ('i',)
    strip = '@class="hidden"'
    snippet = '<p><b><i>Only</i></b> this text remains.<span class="hidden">Not this one.</span></p>'

    result = remove_tags(snippet, allowed_tags=allowed_tags, strip=strip)
    expected = u'<i>Only</i> this text remains.'

    assert result == expected


def test_remove_tags_unicode():
    snippet = u'<p>😋</p>'

    result = remove_tags(snippet)
    expected = u'😋'

    assert result == expected


def test_remove_tags_strip_keeps_tails():
    strip = 'self::foo'
    snippet = '<foo>This goes</foo> but this remains.'

    result = remove_tags(snippet, strip=strip)
    expected = u' but this remains.'

    assert result == expected



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2018 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""ORCID configuration loader."""

from __future__ import absolute_import, division, print_function

import os
import pytest

from inspire_utils.config import Config, load_config, MalformedConfig


@pytest.fixture
def restore_cwd():
    cwd = os.getcwd()
    yield
    os.chdir(cwd)


def test_config(tmpdir):
    mock_config = tmpdir.join("inspirehep.cfg")
    mock_config.write("SERVER_NAME = '0.0.0.0'; OTHER_VARIABLE = 42")

    config = Config(defaults={
        'SERVER_NAME': '127.0.0.1',
        'SOME_OTHER_DEFAULT': 1234,
    })
    config.load_pyfile(mock_config.strpath)

    assert config['SERVER_NAME'] == '0.0.0.0'
    assert config['OTHER_VARIABLE'] == 42
    assert config.get('SOME_OTHER_DEFAULT') == 1234
    assert 'NOT_IN_CONFIG' not in config


def test_config_empty_file(tmpdir):
    mock_config = tmpdir.join("inspirehep.cfg")
    mock_config.write("")

    config = Config()
    config.load_pyfile(mock_config.strpath)

    assert 'NOT_IN_CONFIG' not in config


def test_config_inexistent_file(tmpdir):
    mock_config = tmpdir.join("inspirehep.cfg")
    mock_config.write("FOR_DELETION = 10")
    mock_config.remove()

    config = Config()

    with pytest.raises(IOError):
        config.load_pyfile(mock_config.strpath)


def test_config_invalid_file(tmpdir):
    mock_config = tmpdir.join("inspirehep.cfg")
    mock_config.write("this is py#0n G1|3|3er1sh")

    config = Config()

    with pytest.raises(MalformedConfig):
        config.load_pyfile(mock_config.strpath)


def test_load_config(restore_cwd, tmpdir):
    mock_inspirehep_var_cfg = tmpdir.mkdir('var').mkdir('inspirehep-instance').join("inspirehep.cfg")
    mock_inspirehep_var_cfg.write("SERVER_NAME = '0.0.0.0'")

    mock_inspirehep_cfg = tmpdir.join("inspirehep.cfg")
    mock_inspirehep_cfg.write("SERVER_NAME = '127.0.0.1'; OTHER_VARIABLE = 42")

    os.chdir(tmpdir.strpath)
    config = load_config()

    assert config['SERVER_NAME'] == '127.0.0.1'
    assert config['OTHER_VARIABLE'] == 42



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

from inspire_utils.record import get_value, get_values_for_schema


def test_get_value_returns_all_values():
    record = {
        'titles': [
            {'title': 'first title'},
            {'title': 'second title'},
        ],
    }

    expected = [
        'first title',
        'second title',
    ]
    result = get_value(record, 'titles.title')

    assert expected == result


def test_get_value_allows_indexes_in_paths():
    record = {
        'titles': [
            {'title': 'first title'},
            {'title': 'second title'},
        ],
    }

    expected = 'second title'
    result = get_value(record, 'titles.title[1]')

    assert expected == result


def test_get_value_allows_slices_in_paths():
    record = {
        'titles': [
            {'title': 'first title'},
            {'title': 'second title'},
        ],
    }

    expected = [
        'first title',
        'second title',
    ]
    result = get_value(record, 'titles.title[:]')

    assert expected == result


def test_get_value_returns_none_if_inner_key_does_not_exist_on_string():
    record = {
        'foo': 'bar'
    }

    result = get_value(record, 'foo.value')

    assert result is None


def test_get_value_works_on_lists():
    record = [
        {'foo': 'bar'},
    ]

    expected = ['bar']
    result = get_value(record, 'foo')

    assert expected == result


def test_get_values_for_schema():
    elements = [
        {'schema': 'good', 'value': 'first'},
        {'schema': 'bad', 'value': 'second'},
        {'schema': 'good', 'value': 'third'},
    ]
    assert get_values_for_schema(elements, 'good') == ['first', 'third']



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

import pytest

from inspire_utils.date import (
    PartialDate,
    earliest_date,
    format_date,
    normalize_date,
    fill_missing_date_parts
)


def test_loads_validate_dates_day():
    with pytest.raises(ValueError):
        PartialDate.loads('2015-10-1')


def test_loads_validate_dates_day_when_day_is_00():
    with pytest.raises(ValueError):
        PartialDate.loads('2015-10-00')


def test_loads_validate_dates_month_when_month_is_00():
    with pytest.raises(ValueError):
        PartialDate.loads('2015-00')


def test_loads_validate_dates_month():
    with pytest.raises(ValueError):
        PartialDate.loads('2015-1-10')


def test_partial_date_accepts_valid_dates():
    PartialDate(1686, 6, 30)


def test_partial_date_raises_on_invalid_dates():
    with pytest.raises(ValueError):
        PartialDate(1686, 1, 42)


def test_partial_date_raises_on_day_with_no_month():
    with pytest.raises(TypeError):
        PartialDate(1686, None, 30)


def test_partial_date_raises_on_wrong_types():
    with pytest.raises(TypeError):
        PartialDate('1686', '6', '30')


def test_partial_date_equality():
    assert PartialDate(1686, 6) == PartialDate(1686, 6)


def test_partial_date_self_inequality():
    date = PartialDate(1686)

    assert not date < date


def test_partial_date_sorts_incomplete_dates_after_complete_dates():
    complete = PartialDate(1686, 6, 30)
    incomplete = PartialDate(1686)

    assert complete < incomplete
    assert not incomplete < complete


def test_partial_date_loads():
    expected = PartialDate(1686, 6)

    assert expected == PartialDate.loads('1686-06')


def test_partial_date_from_parts():
    expected = PartialDate(1686, 6)

    assert expected == PartialDate.from_parts(1686, 'June')


def test_partial_date_pprints_when_cast_to_str():
    expected = 'Jun, 1686'

    assert expected == str(PartialDate(1686, 6))


def test_partial_date_pprints_correct_date():
    expected_full = 'Jan 1, 1890'
    expected_no_day = 'Jan, 1890'
    expected_no_month = '1890'

    assert expected_full == PartialDate(1890, 1, 1).pprint()
    assert expected_no_day == PartialDate(1890, 1).pprint()
    assert expected_no_month == PartialDate(1890).pprint()


def test_format_date():
    expected = u'Jun 30, 1686'
    result = format_date('1686-06-30')

    assert expected == result


def test_format_date_incomplete():
    expected = u'Jun, 1686'
    result = format_date('1686-06')

    assert expected == result


def test_normalize_date_handles_ISO():
    expected = '1686-06-30'
    result = normalize_date('1686-06-30')

    assert expected == result


def test_normalize_date_handles_year_only():
    expected = '1686'
    result = normalize_date('1686')

    assert expected == result


def test_normalize_date_handles_year_month():
    expected = '1686-06'
    result = normalize_date('1686-06')

    assert expected == result


@pytest.mark.xfail(reason='Output is wrong as year has less than 4 digits')
def test_normalize_date_handles_default_dates():
    default_date1 = '0001-01-01'
    default_date2 = '0002-02-02'

    assert default_date1 == normalize_date('0001-01-01')
    assert default_date2 == normalize_date('0002-02-02')


def test_normalize_date_handles_human_friendly_dates():
    expected = '1686-06-30'
    result = normalize_date('Fri June 30 1686')

    assert expected == result


def test_normalize_date_raises_on_dates_without_year():
    with pytest.raises(ValueError):
        normalize_date('Fri June 30')


def test_normalize_date_raises_on_unparseable_dates():
    with pytest.raises(ValueError):
        normalize_date('Foo')


def test_normalize_date_returns_none_on_none():
    assert normalize_date(None) is None


def test_earliest_date():
    expected = '1686-06-30'
    result = earliest_date(['1686-06', '1686-06-30'])

    assert expected == result


def test_format_date_only_year_regression_iso_week_misuse():
    expected = u'1993'
    result = format_date('1993')

    assert expected == result


def test_fill_missing_date_parts_adds_day():
    expected = "2019-06-01"
    result = fill_missing_date_parts("2019-06")

    assert expected == result


def test_fill_missing_date_parts_adds_month_and_day():
    expected = "2019-01-01"
    result = fill_missing_date_parts("2019")

    assert expected == result



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

import pytest
from inspire_utils.name import (ParsedName, format_name,
                                generate_name_variations, normalize_name)
from inspire_utils.query import ordered
from mock import patch


def test_normalize_name_full():
    expected = "Smith, John Peter"

    assert expected == normalize_name("Smith, John Peter")


def test_normalize_name_handles_names_with_first_initial():
    expected = "Smith, J. Peter"

    assert expected == normalize_name("Smith, J Peter")
    assert expected == normalize_name("Smith, J. Peter")
    assert expected == normalize_name("Smith, J. Peter ")


def test_normalize_name_handles_names_with_middle_initial():
    expected = "Smith, John P."

    assert expected == normalize_name("Smith, John P.")
    assert expected == normalize_name("Smith, John P. ")
    assert expected == normalize_name("Smith, John P ")


def test_normalize_name_handles_names_with_dots_initials():
    expected = "Smith, J.P."

    assert expected == normalize_name("Smith, J. P.")
    assert expected == normalize_name("Smith, J.P.")
    assert expected == normalize_name("Smith, J.P. ")
    assert expected == normalize_name("Smith, J. P. ")


def test_normalize_name_handles_names_with_spaces():
    expected = "Smith, J.P."

    assert expected == normalize_name("Smith, J P ")
    assert expected == normalize_name("Smith, J P")


def test_normalize_name_handles_names_with_several_last_names():
    expected = "Smith Davis, J.P."

    assert expected == normalize_name("Smith Davis, J.P.")


def test_normalize_name_handles_jimmy():  # http://jimmy.pink
    expected = "Jimmy"

    assert expected == normalize_name("Jimmy")


def test_regression_name_with_comma():
    expected = "Jessica"
    assert expected == normalize_name("Jessica, ")


def test_normalize_name_handles_unicode():
    expected = u"蕾拉"

    assert expected == normalize_name(u"蕾拉")


def test_normalize_name_converts_unicode_apostrophe_to_normal_apostrophe():
    expected = u"M'Gregor, Jimmy"

    assert expected == normalize_name(u"M’Gregor, Jimmy")


@pytest.mark.parametrize(
    "input_author_name,expected",
    [
        ("Smith, John Jr", "Smith, John, Jr."),
        ("Smith, John Jr.", "Smith, John, Jr."),
        ("Smith, John III", "Smith, John, III"),
        ("Smith, John iii", "Smith, John, III"),
        ("Smith, John VIII", "Smith, John, VIII"),
        ("Smith, John viii", "Smith, John, VIII"),
        ("Smith, John IV", "Smith, John, IV"),
        ("Smith, John iv", "Smith, John, IV"),
    ],
)
def test_normalize_name_handles_suffixes(input_author_name, expected):
    assert normalize_name(input_author_name) == expected


@pytest.mark.parametrize(
    "input_author_name,expected",
    [("Sir John Smith", "Smith, John"),
     ("Bao, Hon", "Bao, Hon"), ("ed witten", "Witten, Ed")],
)
def test_normalize_name_handles_titles(input_author_name, expected):
    assert normalize_name(input_author_name) == expected


def test_generate_name_variations_with_two_non_lastnames():
    name = "Ellis, John Richard"
    expected_name_variations = {
        "ellis",
        "ellis j",
        "ellis j r",
        "ellis j richard",
        "ellis john",
        "ellis john r",
        "ellis john richard",
        "ellis r",
        "ellis richard",
        "ellis, j",
        "ellis, j r",
        "ellis, j richard",
        "ellis, john",
        "ellis, john r",
        "ellis, john richard",
        "ellis, r",
        "ellis, richard",
        "j ellis",
        "j r ellis",
        "j richard ellis",
        "john ellis",
        "john r ellis",
        "john richard ellis",
        "r ellis",
        "richard ellis",
        "j, ellis",
        "j r, ellis",
        "j richard, ellis",
        "john, ellis",
        "john r, ellis",
        "john richard, ellis",
        "r, ellis",
        "richard, ellis",
    }

    result = generate_name_variations(name)

    assert set(result) == expected_name_variations


def test_generate_name_variations_with_more_than_two_non_lastnames_does_not_add_extra_spaces():
    name = "Ellis, John Richard Philip"

    result = generate_name_variations(name)

    assert "ellis, john  philip" not in set(result)


def test_generate_name_variations_with_two_lastnames():
    name = u"Caro Estevez, David"
    expected = {
        # Lastnames only
        u"caro",
        u"estevez",
        u"caro estevez",
        # Lastnames first and then non lastnames
        u"caro estevez d",
        u"caro estevez david",
        u"caro estevez, d",
        u"caro estevez, david",
        u"caro d",
        u"caro, d",
        u"caro david",
        u"caro, david",
        u"estevez d",
        u"estevez david",
        u"estevez, d",
        u"estevez, david",
        # Non lastnames first and then lastnames
        u"d caro",
        u"d, caro",
        u"d estevez",
        u"d, estevez",
        u"d caro estevez",
        u"d, caro estevez",
        u"david caro",
        u"david, caro",
        u"david estevez",
        u"david, estevez",
        u"david caro estevez",
        u"david, caro estevez",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_with_three_lastnames_dashed_ignores_the_dash():
    name = u"Caro-Estévez Martínez, David"
    expected = {
        # Lastnames only
        u"caro",
        u"estevez",
        u"martinez",
        u"caro estevez martinez",
        # Lastnames first and then non lastnames
        u"caro estevez martinez d",
        u"caro estevez martinez david",
        u"caro estevez martinez, d",
        u"caro estevez martinez, david",
        u"caro d",
        u"caro, d",
        u"caro david",
        u"caro, david",
        u"estevez d",
        u"estevez, d",
        u"estevez david",
        u"estevez, david",
        u"martinez d",
        u"martinez, d",
        u"martinez david",
        u"martinez, david",
        # Non lastnames first and then lastnames
        u"d caro",
        u"d, caro",
        u"d estevez",
        u"d, estevez",
        u"d martinez",
        u"d, martinez",
        u"d caro estevez martinez",
        u"d, caro estevez martinez",
        u"david caro",
        u"david, caro",
        u"david estevez",
        u"david, estevez",
        u"david martinez",
        u"david, martinez",
        u"david caro estevez martinez",
        u"david, caro estevez martinez",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_with_firstname_as_initial():
    name = "Smith, J"
    expected = {
        # Lastname only
        u"smith",
        # Lastnames first and then non lastnames
        u"smith j",
        u"smith, j",
        # Non lastnames first and then lastnames
        u"j smith",
        u"j, smith",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_with_only_one_name():
    name = "Jimmy"
    expected = {
        u"jimmy",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_with_many_names_defers_generating_variations():
    import logging

    logger = logging.getLogger("inspire_utils.name")
    with patch.object(logger, "warning") as mock_warning:
        many_names_as_one_author = "Tseng, Farrukh Azfar Todd Huffman Thilo Pauly"

        result = generate_name_variations(many_names_as_one_author)

        assert result == [many_names_as_one_author]

        args, _ = mock_warning.call_args
        assert args[0].startswith(
            "Skipping name variations generation - too many names"
        )


def test_generate_name_variations_capitalizes_first_letters():
    name = "mele, salvatore"
    expected = {
        # Lastname only
        u"mele",
        # Lastnames first and then non lastnames
        u"mele s",
        u"mele, s",
        u"mele salvatore",
        u"mele, salvatore",
        # Non lastnames first and then lastnames
        u"salvatore mele",
        u"salvatore, mele",
        u"s mele",
        u"s, mele",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_works_with_two_consecutive_commas():
    name = "Perelstein,, Maxim"
    expected = {
        # Lastname only
        u"perelstein",
        # Lastnames first and then non lastnames
        u"perelstein m",
        u"perelstein, m",
        u"perelstein maxim",
        u"perelstein, maxim",
        # Non lastnames first and then lastnames
        u"maxim perelstein",
        u"maxim, perelstein",
        u"m perelstein",
        u"m, perelstein",
    }

    result = generate_name_variations(name)

    assert set(result) == expected


def test_generate_name_variations_with_short_lastname_and_initial():
    # Should not output something like `o y` or any similar variation.
    name = "Oz, Y"
    expected = {
        # Lastname only
        u"oz",
        # Lastnames first and then non lastnames
        u"oz y",
        u"oz, y",
        # Non lastnames first and then lastnames
        u"y oz",
        u"y, oz",
    }

    result = generate_name_variations(name)

    assert len(result) == len(expected)

    assert set(result) == expected


def test_parsed_name_from_parts():
    parsed_name = ParsedName.from_parts("John", "Smith", "Peter", "Jr", "Sir")

    expected = "Smith, John Peter, Jr."
    result = parsed_name.dumps()

    assert result == expected


def test_normalize_name_handles_multiple_middle_names():
    expected = "Almeida, C.A.S."

    assert expected == normalize_name("Almeida, C. A. S.")
    assert expected == normalize_name("Almeida, C. A.S.")
    assert expected == normalize_name("Almeida, C.A. S.")
    assert expected == normalize_name("Almeida, C.A.S.")


def test_normalize_name_handles_multiple_middle_names_with_and_without_initials():
    expected = "Smith, J.A. Peter J."

    assert expected == normalize_name("Smith, J. A. Peter J.")
    assert expected == normalize_name("Smith, J.A. Peter J.")


def test_format_author_name():
    expected = "Stanley Martin Lieber"

    assert expected == format_name("Lieber, Stanley Martin")

    expected = "Robert Downey, Jr."

    assert expected == format_name("Downey, Robert Jr.")


def test_format_author_name_with_initials():
    expected = "S. M. Lieber"

    assert expected == format_name(
        "Lieber, Stanley Martin", initials_only=True)


def test_format_author_name_with_initials_with_all_caps_name():
    expected = "T. S. Holland"

    assert expected == format_name("HOLLAND, TOM STANLEY", initials_only=True)


def test_parsed_name_initials():
    parsed_name = ParsedName("Holland, Tom Stanley")
    expected = "T. S."

    assert expected == parsed_name.first_initials

    expected = ["T.", "S."]

    assert expected == parsed_name.first_initials_list


@pytest.mark.parametrize(
    "input_author_name,expected",
    [("Lieber, Ed", "E. Lieber"),
     ('Lieber, Ed Viktor', "E. V. Lieber"),
     ('Lieber, Ed Jr.', "E. Lieber, Jr."),
     ('Lieber, Ed Victor Jr.', "E. V. Lieber, Jr."),
     ],
)
def test_format_author_name_with_initials_when_first_name_is_similar_to_title(input_author_name, expected):

    assert expected == format_name(input_author_name, initials_only=True, without_titles=True)


def test_parsed_wrong_names_and_not_fail():
    names = [
        (u"Proffesor.M.", u"Proffesor.M."),
        (u"ˇ Sirˇ", u"Sirˇ, ˇ."),
    ]

    for name, expected in names:
        assert ParsedName(name).dumps() == expected


def test_unicode_characters_in_format_name():
    assert format_name("Cañas, Ramón") == u"Ramón Cañas"
    assert format_name("Süß, Jörg") == u"Jörg Süß"
    assert format_name("Møller, Kyösti") == u"Kyösti Møller"
    assert format_name("Varejão, François") == u"François Varejão"


def test_first_names_are_never_printed_with_initials_only_if_no_last_name():
    expected = u"Jimmy"

    assert expected == format_name("Jimmy", initials_only=True)


def test_first_name_with_dash_is_initialized_correctly():
    assert u"Z. Y. Yin" == format_name("Zhao-Yu Yin", initials_only=True)


def test_first_name_with_dash_is_printed_with_dash_and_initialized_correctly():
    expected = "Huang-Wei Pan"
    parsed_name = ParsedName.loads(expected)
    result = " ".join(parsed_name.first_list + parsed_name.last_list)
    assert expected == result


def test_first_name_initials_without_whitespace_is_initialized_correctly():
    assert u"M. A. M. G. Garcia" == format_name(
        "Miguel A-M.G. Garcia", initials_only=True
    )


def test_last_name_recognized_correctly_regression_test():
    assert u"De Sousa Vieira" == ParsedName.loads("De Sousa Vieira, M.C.").last


def test_generate_es_query_lastname_firstname_with_commas_and_initials():
    name = "ellis, john k."

    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "authors.first_name": {
                                                            "analyzer": "names_analyzer",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "authors.first_name": {
                                                            "analyzer": "names_initials_analyzer",
                                                            "operator": "AND",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_first_author_lastname_firstname_with_commas_and_initials():
    name = "ellis, john k."

    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_analyzer",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_initials_analyzer",
                                                            "operator": "AND",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_query_author_with_composite_last_name():
    name = "de Rham"
    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "query": "Rham",
                                    "operator": "AND",
                                }
                            }
                        },
                        {
                            "bool": {
                                "should": [
                                    {
                                        "match_phrase_prefix": {
                                            "authors.first_name": {
                                                "query": "de",
                                                "analyzer": "names_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name": {
                                                "query": "de",
                                                "operator": "AND",
                                                "analyzer": "names_initials_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.full_name": {
                                                "query": "de Rham",
                                                "operator": "AND",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_split_fa_initial_and_firstname_without_a_space():
    name = "D.John Smith"
    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Smith",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "match": {
                                            "first_author.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "D",
                                            }
                                        }
                                    },
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_analyzer",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_initials_analyzer",
                                                            "operator": "AND",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_hack_to_split_two_initials_without_a_space():
    name = "D.K. Smith"
    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "operator": "AND",
                                    "query": "Smith",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "match": {
                                            "authors.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "D",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("authors")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_split_fa_two_initials_without_a_space():
    name = "D.K. Smith"
    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Smith",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "match": {
                                            "first_author.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "D",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_lastname_initial():
    name = "ellis, j"
    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "match": {
                                "authors.first_name.initials": {
                                    "analyzer": "names_initials_analyzer",
                                    "operator": "AND",
                                    "query": "J",
                                }
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_first_author_lastname_initial():
    name = "ellis, j"
    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "match": {
                                "first_author.first_name.initials": {
                                    "analyzer": "names_initials_analyzer",
                                    "operator": "AND",
                                    "query": "J",
                                }
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_lastname_firstname():
    name = "ellis, john"

    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "should": [
                                    {
                                        "match_phrase_prefix": {
                                            "authors.first_name": {
                                                "analyzer": "names_analyzer",
                                                "query": "John",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "John",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_first_author_lastname_firstname():
    name = "ellis, john"

    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "should": [
                                    {
                                        "match_phrase_prefix": {
                                            "first_author.first_name": {
                                                "analyzer": "names_analyzer",
                                                "query": "John",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.first_name": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "John",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_lastname_firstname_without_comma():
    name = "john ellis"

    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "query": "Ellis",
                                    "operator": "AND",
                                }
                            }
                        },
                        {
                            "bool": {
                                "should": [
                                    {
                                        "match_phrase_prefix": {
                                            "authors.first_name": {
                                                "query": "John",
                                                "analyzer": "names_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name": {
                                                "query": "John",
                                                "operator": "AND",
                                                "analyzer": "names_initials_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.full_name": {
                                                "query": "John Ellis",
                                                "operator": "AND",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_first_author_lastname_firstname_without_comma():
    name = "john ellis"

    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "query": "Ellis",
                                    "operator": "AND",
                                }
                            }
                        },
                        {
                            "bool": {
                                "should": [
                                    {
                                        "match_phrase_prefix": {
                                            "first_author.first_name": {
                                                "query": "John",
                                                "analyzer": "names_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.first_name": {
                                                "query": "John",
                                                "operator": "AND",
                                                "analyzer": "names_initials_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.full_name": {
                                                "query": "John Ellis",
                                                "operator": "AND",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_lastname_firstname_without_commas_and_initials():
    name = "john k. ellis"

    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "authors.first_name": {
                                                            "analyzer": "names_analyzer",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "authors.first_name": {
                                                            "analyzer": "names_initials_analyzer",
                                                            "operator": "AND",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                    {
                                        "match": {
                                            "authors.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_first_author_lastname_firstname_without_commas_and_initials():
    name = "john k. ellis"

    expected_query = {
        "nested": {
            "path": "first_author",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "first_author.last_name": {
                                    "operator": "AND",
                                    "query": "Ellis",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_analyzer",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "first_author.first_name": {
                                                            "analyzer": "names_initials_analyzer",
                                                            "operator": "AND",
                                                            "query": "John",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                    {
                                        "match": {
                                            "first_author.first_name.initials": {
                                                "analyzer": "names_initials_analyzer",
                                                "operator": "AND",
                                                "query": "K",
                                            }
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query("first_author")
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_works_for_unambigous_names():
    name = "J. David Ellis"

    expected_query = {
        "nested": {
            "path": "authors",
            "query": {
                "bool": {
                    "must": [
                        {
                            "match": {
                                "authors.last_name": {
                                    "query": "Ellis",
                                    "operator": "AND",
                                }
                            }
                        },
                        {
                            "bool": {
                                "must": [
                                    {
                                        "match": {
                                            "authors.first_name.initials": {
                                                "query": "J",
                                                "operator": "AND",
                                                "analyzer": "names_initials_analyzer",
                                            }
                                        }
                                    },
                                    {
                                        "bool": {
                                            "should": [
                                                {
                                                    "match_phrase_prefix": {
                                                        "authors.first_name": {
                                                            "query": "David",
                                                            "analyzer": "names_analyzer",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "authors.first_name": {
                                                            "query": "David",
                                                            "operator": "AND",
                                                            "analyzer": "names_initials_analyzer",
                                                        }
                                                    }
                                                },
                                                {
                                                    "match": {
                                                        "authors.full_name": {
                                                            "query": "J. David Ellis",
                                                            "operator": "AND",
                                                        }
                                                    }
                                                },
                                            ]
                                        }
                                    },
                                ]
                            }
                        },
                    ]
                }
            },
        }
    }

    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_generate_es_query_title_name():
    name = "ed witten"
    expected_query = {
        'nested': {
            'path': 'authors', 'query': {
                'bool': {
                    'must': [
                        {
                            'match': {
                                u'authors.last_name': {
                                    'operator': 'AND',
                                    'query': u'Witten'
                                }
                            }
                        }, {
                            'bool': {
                                'should': [
                                    {
                                        'match_phrase_prefix': {
                                            u'authors.first_name': {
                                                'query': u'Ed',
                                                'analyzer': 'names_analyzer'
                                            }
                                        }
                                    }, {
                                        'match': {
                                            u'authors.first_name': {
                                                'operator': 'AND',
                                                'query': u'Ed',
                                                'analyzer': 'names_initials_analyzer'
                                            }
                                        }
                                    }, {
                                        'match': {
                                            u'authors.full_name': {
                                                'operator': 'AND',
                                                'query': 'Ed Witten'
                                            }
                                        }
                                    }
                                ]
                            }
                        }
                    ]
                }
            }
        }
    }
    parsed_name = ParsedName(name)
    generated_es_query = parsed_name.generate_es_query()
    assert ordered(generated_es_query) == ordered(expected_query)


def test_parsed_name_doesnt_produce_error_when_first_and_last_name_empty():
    empty_parsed_name = ParsedName("")
    parsed_name_with_title_only = ParsedName("Editor")
    assert empty_parsed_name is not None
    assert parsed_name_with_title_only is not None


def test_dot_is_not_a_name():
    name_with_dot = ParsedName("Meng, .")
    assert not name_with_dot.first
    assert not name_with_dot.first_list



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2018 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Tests for URL utils."""

from __future__ import absolute_import, division, print_function

import pytest

from inspire_utils.urls import ensure_scheme, record_url_by_pattern


@pytest.mark.parametrize(
    'url,scheme,expected',
    [
        ('http://inspirehep.net', None, 'http://inspirehep.net'),
        ('http://inspirehep.net', 'https', 'http://inspirehep.net'),
        ('inspirehep.net', None, 'http://inspirehep.net'),
        ('inspirehep.net', 'custom+http', 'custom+http://inspirehep.net'),
        (u'ïnśpirę.nẽt', 'https', u'https://ïnśpirę.nẽt'),
        ('http://inspirehep.net/path', None, 'http://inspirehep.net/path'),
        ('inspirehep.net/path', None, 'http://inspirehep.net/path'),
        ('//inspirehep.net/path', None, 'http://inspirehep.net/path'),
        ('/path/somewhere', None, 'http:///path/somewhere'),
        ('http:///path/somewhere', None, 'http:///path/somewhere'),
    ],
    ids=[
        'scheme already present',
        'scheme present, ignores default',
        'add default scheme',
        'add custom scheme',
        'unicode',
        'with scheme and with path',
        'no scheme, with path',
        'explicit netloc',
        'with scheme, no netloc',
        'without scheme, no netloc',
    ]
)
def test_ensure_scheme(url, scheme, expected):
    if scheme:
        assert ensure_scheme(url, default_scheme=scheme) == expected
    else:
        assert ensure_scheme(url) == expected


@pytest.mark.parametrize(
    'pattern,recid,expected',
    [
        (
            'http://inspirehep.net/record/{recid}',
            123,
            'http://inspirehep.net/record/123',
        ),
        (
            'inspirehep.net/record/{recid}',
            '4567',
            'http://inspirehep.net/record/4567',
        ),
        (
            u'http://ïnśpirę.nẽt/record/{recid}',
            123,
            u'http://ïnśpirę.nẽt/record/123',
        ),
    ],
    ids=[
        'integer recid, scheme already present',
        'string recid, no scheme',
        'unicode url',
    ]
)
def test_record_url_by_pattern(pattern, recid, expected):
    assert record_url_by_pattern(pattern, recid) == expected



# -*- coding: utf-8 -*-
#
# This file is part of INSPIRE.
# Copyright (C) 2014-2017 CERN.
#
# INSPIRE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# INSPIRE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with INSPIRE. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import absolute_import, division, print_function

import six


def dedupe_list(list_with_duplicates):
    """Remove duplicates from a list preserving the order.

    We might be tempted to use the list(set(l)) idiom, but it doesn't preserve
    the order, which hinders testability.
    """
    result = []

    for el in list_with_duplicates:
        if el not in result:
            result.append(el)

    return result


def dedupe_list_of_dicts(ld):
    """Remove duplicates from a list of dictionaries preserving the order.

    We can't use the generic list helper because a dictionary isn't hashable.
    Adapted from http://stackoverflow.com/a/9427216/374865.
    """
    def _freeze(o):
        """Recursively freezes a dict into an hashable object.

        Adapted from http://stackoverflow.com/a/21614155/374865.
        """
        if isinstance(o, dict):
            return frozenset((k, _freeze(v)) for k, v in six.iteritems(o))
        elif isinstance(o, (list, tuple)):
            return tuple(_freeze(v) for v in o)
        else:
            return o

    result = []
    seen = set()

    for d in ld:
        f = _freeze(d)
        if f not in seen:
            result.append(d)
            seen.add(f)

    return result


def dedupe_all_lists(obj, exclude_keys=(), dedupe_top_level=True):
    """Recursively remove duplicates from all lists.

    Args:
        obj: collection to deduplicate
        exclude_keys (Container[str]): key names to ignore for deduplication
        dedupe_top_level (bool): whether the top-level list should be deduplicated too
    """
    squared_dedupe_len = 10
    if isinstance(obj, dict):
        new_obj = {}
        for key, value in obj.items():
            dedupe_top_level = key not in exclude_keys
            new_obj[key] = dedupe_all_lists(value, exclude_keys, dedupe_top_level)
        return new_obj
    elif isinstance(obj, (list, tuple, set)):
        new_obj = [dedupe_all_lists(v, exclude_keys) for v in obj]
        if dedupe_top_level:
            if len(new_obj) < squared_dedupe_len:
                new_obj = dedupe_list(new_obj)
            else:
                new_obj = dedupe_list_of_dicts(new_obj)
        return type(obj)(new_obj)
    else:
        return obj
